open YaccTypes;;
open PrinterYacc;;
open PrinterLex;;
open MakeSVG;;
open PrintSVG;;

let evaluate () =
 begin
  let file = ref "" in
  let surpMin = ref 0.0 in
  let surpMax = ref 100.0 in
  let onePageString = ref "true" in
  let textSize = ref "normal" in
  Arg.parse [ ("-file",Arg.Set_string file,"Input file in proper format (see README for specs)");
              ("-surpMin", Arg.Set_float surpMin, "Minimum float value for surprisal printing. Default is 0.0");
              ("-surpMax", Arg.Set_float surpMax, "Maximum float value for surprisal printing. Default is 100.0");
              ("-oneFile", Arg.Set_string onePageString, "Print all sentences in one file, rather than have each sentence print into individual files. Default is true");
	      ("-textSize", Arg.Set_string textSize, "Set the size of the output (\"small\", \"normal\", or \"large\" for poster-size). Default is normal");]
    (function s -> print_string ("didn't understand "^s^"\n"))
    "Invoke the dependency printer";

  if !file = "" then failwith "input file required";
  let onePage = if ((compare !onePageString "true")==0) then ref true else ref false in 

  let sentenceList =
  begin
    YaccTypes.linenum := 1; 
    try PrinterYacc.sentenceList PrinterLex.token (Lexing.from_channel (open_in !file))
    with (Parsing.Parse_error ) ->
      (print_string ("Syntax error in line "^(string_of_int !YaccTypes.linenum)^"\n");
      [])
  end in
  let svgList = run sentenceList !surpMin !surpMax !onePage !textSize in
  let runIt = (runSVGPrint svgList !onePage !textSize) in 
    if (runIt == true) then (print_string ("Finished reading "^(!file)^", "^(string_of_int ((List.length sentenceList)-1))^" sentences printed.\n")) else (print_string ("Something didn't work\n"));

end;;

evaluate();;
