open YaccTypes;;


(*REMOVE THESE*)

(*UTILITIES*)


type arrow = {id: string; fill: string};;
type path = {start: point; finish: point; reverse: bool; pid:string};;
type arcPath = {path: path; pcolor: string; pstyle: string; endMarker: string};;
type arcTitle = {ttitle: string; tid: string; tcolor: string};;
type surprisal = {width: float; sstart: point; stext: string};;
type svg = {words: word list; arrowList: arrow list; arcPathList: arcPath list; arcTitleList: arcTitle list; surprisalList: surprisal list};;
type textSize = Small | Normal | Large
type textNumbers = {name: textSize; perPage: int; sentSpace: float; xSpace: float; ySpace: float; surprisalSize: float; charPixels: float; aboveWord: float}


let pageWidth=468.0;;
let pageHeight=648.0;;
let startPosition = {x=72.0;y=200.0};;
let endPosition = {x=(startPosition.x +. pageWidth); y=(startPosition.y +. pageHeight)};;


(*Make Word Data*)
let rec findArcs word arcList= match arcList with
    [] -> word
  |h::t -> 
     if ((h.fromWord == word.pos) || (h.toWord == word.pos))
    then (findArcs {wordInfo=word.wordInfo; pos=word.pos; startPoint=word.startPoint; endPoint=word.endPoint; arcs=(word.arcs + 1); arcPoint=word.arcPoint; surprisal=word.surprisal} t)
     else (findArcs word t);;

let rec numberOfArcs wordList arcList newWordList = match wordList with
    [] -> (List.rev newWordList)
  |h::t -> numberOfArcs t arcList ((findArcs h arcList)::newWordList);;

let rec findMax wi i = match wi with
    [] -> (float_of_int i)
  |h::t -> if ((String.length h) > i) then (findMax t (String.length h)) else (findMax t i);;

let findArcPoints a mid= match a with
    1 -> mid
  |s -> match (s mod 2) with
	 0 -> let d = (float_of_int (s/2*3)) in {x=((mid.x +. 1.5) -. d); y=mid.y}
       |_ -> let d = (float_of_int (s/2*3)) in {x=(mid.x -. d); y=mid.y};;

let rec findWordPoints wl mid s e i textNos = match wl with
    [] -> (List.rev s), (List.rev e)
  |h::t -> let pix = ((float_of_int (String.length h)) *. textNos.charPixels) in
    let half = (pix /. 2.0) in
    let y = if (i = 0) then (mid.y) else (mid.y +. ((float_of_int i)*.textNos.ySpace)) in 
    let st = {x=(mid.x -. half); y=y} in
    let en = {x=(mid.x +. half); y=y} in
      (findWordPoints t mid (st::s) (en::e) (i+1) textNos);;

let findPoints w start textNos= 
  let len = ((findMax w.wordInfo 0) *. textNos.charPixels) in
  let final = {x=start.x +. len;y=start.y} in
  let midPoint = {x=(start.x +. ((final.x -. start.x)/.2.0)); y=start.y} in
  let arcPoint = (findArcPoints w.arcs midPoint) in
  let startPoints, endPoints = (findWordPoints w.wordInfo midPoint [] [] 0 textNos) in
    {wordInfo=w.wordInfo; pos=w.pos; startPoint=startPoints; endPoint=endPoints; arcs=w.arcs; arcPoint=arcPoint; surprisal=w.surprisal}, {x=final.x +. textNos.xSpace;y=final.y};;
    

let rec makePoints wl nwl start textNos= match wl with
    [] -> (List.rev nwl)
  |h::t -> let nw, newStart = (findPoints h start textNos) in
	     (makePoints t (nw::nwl) newStart textNos);;

let rec makeWordData wl al newWL textNos pos  = (makePoints (numberOfArcs wl al []) [] pos textNos);;

(*Make Path Data*)

let rec makeMidList wl nl = match wl with
    [] -> (List.rev nl)
  |h::t -> (makeMidList t (h.arcPoint::nl));;

let makeMidArray wl =
  let rec makeMA wl a = match wl with
      [] -> a
    |h::t -> ((Array.set a h.pos h.arcPoint); (makeMA t a))
  in (makeMA wl (Array.make (List.length wl) newPoint));; 

let rec arrowId arrl col = match arrl with
    [] -> (List.hd arrl).id
  |h::t -> if ((compare h.fill col) == 0) then h.id else (arrowId t col);;

let makeOnePath wl a arrl midArray i textNos = 
  let startPoint = (Array.get midArray a.fromWord) in
  let endPoint = (Array.get midArray a.toWord) in
  let reverse = (if (startPoint > endPoint) then false else true) in
  let id = ("path"^(string_of_int i)^"-"^(string_of_int a.fromWord)^"-"^(string_of_int a.toWord)) in 
  let path = {start={x=startPoint.x;y=(startPoint.y -. textNos.aboveWord)}; finish={x=endPoint.x;y=(endPoint.y -. textNos.aboveWord)}; reverse=reverse; pid=id} in
  let arrow = (arrowId arrl a.color) in
  let arcPath = {path=path; pcolor=a.color; pstyle=a.style; endMarker=arrow} in 
  let arcTitle = {ttitle=a.title; tid=id; tcolor=a.color} in
    arcPath, arcTitle;;

let rec updateArray a l = match l with
    [] -> a
  |h::t -> let oldOne = Array.get a h in
	     Array.set a h {x=(oldOne.x +. 4.0);y=(oldOne.y)}; (updateArray a t);;


let rec makePathData wl al arrl midArray newAPL newATL i textNos = match al with
    [] -> (List.rev newAPL), (List.rev newATL)
  |h::t -> let ap, at = (makeOnePath wl h arrl midArray i textNos) in
	     makePathData wl t arrl (updateArray midArray [h.fromWord;h.toWord]) (ap::newAPL) (at::newATL) i textNos;;



(*MakeArrowData*)

let rec checkArrow arc arrList i j = match arrList with
    [] -> [{id=("arrow"^(string_of_int j)^"-"^(string_of_int i)); fill=arc.color}]
  |h::t -> if ((compare arc.color h.fill) == 0) then [] else (checkArrow arc t i j);;

let rec makeArrows al nl i j = match al with
    [] -> nl
  |h::t -> match (checkArrow h nl i j) with
	[] -> (makeArrows t nl i j)
      |hh::tt -> (makeArrows t (hh::nl) (i+1) j);;

(*RESET ARC ENDPOINTS*)
type reset = {changePoint: point; distance:float; changeType: string; wordArc: word; pathChanged: arcPath};;

let rec goThroughArcs w apl nl pArray= match apl with
    [] -> (Array.set pArray w.pos (List.rev nl)); pArray
  |hh::tt -> 
     let s = (List.hd w.startPoint).x in
     let e = (List.hd w.endPoint).x in
       if ((hh.path.start.x > s) && (hh.path.start.x < e))
       then (goThroughArcs w tt ({changePoint=hh.path.start;changeType="START";wordArc=w;pathChanged=hh;distance=(abs_float (hh.path.finish.x -. hh.path.start.x))}::nl) pArray)
       else 
	   (if ((hh.path.finish.x > s) && (hh.path.finish.x < e)) 
	   then (goThroughArcs w tt ({changePoint=hh.path.finish;changeType="FINISH";wordArc=w;pathChanged=hh;distance=(abs_float (hh.path.finish.x -. hh.path.start.x))}::nl) pArray)
	   else (goThroughArcs w tt nl pArray));;

let organizePaths apl wl =
  let rec ops apl wl pArray = match wl with
      [] -> pArray
    |h::t -> (ops apl t (goThroughArcs h apl [] pArray))  
  in (ops apl wl (Array.make (List.length wl) []));;

let rec getCPs l n = match l with
    [] -> n
  |h::t -> getCPs t (h::n);;

let rec op h ol nl = match ol with
    [] -> (List.rev (h.changePoint::nl))
  |hh::tt -> 
     if (h.changePoint.x > hh.x) 
     then (op h tt (hh::nl)) 
     else (List.rev ((getCPs tt [])@(hh::(h.changePoint::nl))));;

let rec orderPoints pl nl = match pl with
    [] -> nl
  |h::t -> orderPoints t (op h nl []);;

let rec fo h ol nl = match ol with
    [] -> (List.rev (h::nl))
  |hh::tt -> 
     if (h.distance > hh.distance) 
     then (fo h tt (hh::nl)) 
     else (List.rev ((List.rev tt)@(hh::(h::nl))));;

let rec findOrder pl nl = match pl with
    [] -> nl
  |h::t -> findOrder t (fo h nl []);;

let rec allButLast pl nl = match pl with
    [] -> (List.rev nl)
  |h::[] -> (List.rev nl)
  |hh::t -> allButLast t (hh::nl);;

let rec orderPaths pathl pointl nl = match pathl with
    [] -> nl
  |h::t -> match h.changeType with
	"START" ->
	  if (h.pathChanged.path.start > h.pathChanged.path.finish)
	  then (orderPaths t (List.tl pointl) ({changePoint=(List.hd pointl);changeType=h.changeType;distance=h.distance;wordArc=h.wordArc;pathChanged=h.pathChanged}::nl))
	  else  (orderPaths t (allButLast pointl []) ({changePoint=(List.nth pointl ((List.length pointl) - 1));changeType=h.changeType;distance=h.distance;wordArc=h.wordArc;pathChanged=h.pathChanged}::nl))
      |"FINISH" ->
	 if (h.pathChanged.path.start > h.pathChanged.path.finish)
	 then (orderPaths t (allButLast pointl []) ({changePoint=(List.nth pointl ((List.length pointl) - 1));changeType=h.changeType;distance=h.distance;wordArc=h.wordArc;pathChanged=h.pathChanged}::nl))
	 else  (orderPaths t (List.tl pointl) ({changePoint=(List.hd pointl);changeType=h.changeType;distance=h.distance;wordArc=h.wordArc;pathChanged=h.pathChanged}::nl))
      |_ -> (orderPaths t pointl nl);;

let rec findDuplicate x y = match y with
    [] -> (match x.changeType with
	"START" -> x.pathChanged.path.finish
      |"FINISH" -> x.pathChanged.path.start
      |_ -> x.pathChanged.path.start)
  |h::t -> if ((compare x.pathChanged.path.pid h.pathChanged.path.pid) == 0) then h.changePoint else (findDuplicate x t);;

let rec removeList x l nl = match l with
    [] -> (List.rev nl)
  |h::t -> if ((compare x.pathChanged.path.pid h.pathChanged.path.pid) == 0) then ((List.rev nl)@t) else (removeList x t (h::nl));;

let rec removeDuplicates rl apl = match rl with
    [] -> apl
  |h::t -> let other = (findDuplicate h t) in
	     match h.changeType with 
		 "START" -> (removeDuplicates (removeList h t []) ({path={start=h.changePoint;finish=other;pid=h.pathChanged.path.pid;reverse=h.pathChanged.path.reverse};pcolor=h.pathChanged.pcolor;pstyle=h.pathChanged.pstyle;endMarker=h.pathChanged.endMarker}::apl))
	       |"FINISH" -> (removeDuplicates (removeList h t []) ({path={start=other;finish=h.changePoint;pid=h.pathChanged.path.pid;reverse=h.pathChanged.path.reverse};pcolor=h.pathChanged.pcolor;pstyle=h.pathChanged.pstyle;endMarker=h.pathChanged.endMarker}::apl))
	       |_ -> removeDuplicates t apl;;

let rec reSetEnds pArray nl i = match ((Array.length pArray) > i) with
    false -> (removeDuplicates nl [])
  |true -> match (List.length (pArray.(i))) with
	0 -> (reSetEnds pArray nl (i+1))
      |_ ->  (reSetEnds pArray (nl@(orderPaths (findOrder pArray.(i) []) (orderPoints pArray.(i) []) [])) (i+1));;

(*MAKE SURPRISAL*)
let rec findSurpX spl soFar = match spl with
    [] -> (soFar)
  |h::t -> if (h.x < soFar) then (findSurpX t h.x) else (findSurpX t soFar);;

let rec findSurpY spl soFar textNos = match spl with
    [] -> (soFar +. textNos.ySpace)
  |h::t -> if (h.y > soFar) then (findSurpY t h.y textNos) else (findSurpY t soFar textNos);;

let findWidth surp min max textNos = 
  let total = (max -. min) in
  let percent = (if (total > surp) then (surp /. total) else (surp /. 100.0)) in
  let barSize = textNos.surprisalSize in
    (barSize *. percent);;

let rec makeSurprisals wl sl smin smax textNos = match wl with
    [] -> (List.rev sl)
  |h::t -> if (h.surprisal > 0.0) then (
     let yy = (findSurpY h.startPoint 0.0 textNos) in 
     let xx = (findSurpX h.startPoint 1000.0) in
     let start = {x=xx;y=yy} in
     let text = (try (String.sub (string_of_float h.surprisal) 0 5) with Invalid_argument "String.sub" -> (string_of_float h.surprisal)) in
     let width = (findWidth h.surprisal smin smax textNos) in 
       (makeSurprisals t ({width=width; sstart=start; stext=text}::sl) smin smax textNos)) else (makeSurprisals t sl smin smax textNos);;

(*MAIN*)

let mainSVG h pos i smin smax textNos= 
  let wordList = (makeWordData h.wordList h.arcList [] textNos pos) in
  let arrowList = (makeArrows h.arcList [] 0 i) in
  let arcPathList, arcTitleList = (makePathData wordList h.arcList arrowList (makeMidArray wordList) [] [] i textNos) in
  let newAPL = (reSetEnds (organizePaths arcPathList wordList) [] 0) in 
  let surprisals = (makeSurprisals wordList [] smin smax textNos) in
    {words = wordList; arrowList=arrowList; arcPathList=newAPL; arcTitleList=arcTitleList; surprisalList=surprisals};;
    
let run sentenceList smin smax onePage textSize =
  let textNos = (match textSize with
      "small" -> {name=Small; perPage=4; sentSpace=200.0; xSpace=8.0; ySpace=15.0; surprisalSize=25.0; charPixels=6.0; aboveWord=15.0}
    |"large" -> {name=Large; perPage=2; sentSpace=400.0; xSpace=20.0; ySpace=30.0; surprisalSize=60.0; charPixels=16.0; aboveWord=30.0}
    |_ -> {name=Normal; perPage=4; sentSpace=200.0; xSpace=10.0; ySpace=15.0; surprisalSize=30.0; charPixels=8.0; aboveWord=15.0}) in
  let rec runIt sentenceList svgList position i textNos = match sentenceList with
      [] -> (List.rev svgList)
    |h::t -> 
       let add = if (((i mod textNos.perPage) == 0) && (i > 2)) then 600.0 else textNos.sentSpace in
      let p = if (onePage == true) then {x=(position.x);y=(position.y +. add)} else {x=(position.x);y=(position.y)} in 
	       runIt t ((mainSVG h position i smin smax textNos)::svgList) p (i+1) textNos
  in (runIt sentenceList [] startPosition 0 textNos);;



