open YaccTypes;;
open MakeSVG;;

type textPrint = {font: string; surpFont: string; surpWidth: string; titleFont: string; arrowW: string; arrowH: string; stroke: string};;

let makeSVG i = (open_out ("../Output/sentence"^i^".svg"));;

let header = 
  "<?xml version=\"1.0\" standalone=\"no\"?>
<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">
<svg width=\"100%\" height=\"100%\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\">
\t<defs>\n";;

let rec printArrows p al textNos = match al with 
    [] -> flush p
  |h::t -> output_string p ("\t\t<marker id=\""^h.id^"\" viewBox=\"0 0 10 10\" refX=\"1\" refY=\"5\" markerUnits=\"strokeWidth\" orient=\"auto\" markerWidth=\""^textNos.arrowW^"\" markerHeight=\""^textNos.arrowH^"\">
\t\t\t<polyline points=\"0,0 10,5 0,10 1,5\" fill=\""^h.fill^"\" />
\t\t</marker>\n"); printArrows p t textNos;;

let rec printPaths p l = match l with
    [] -> (output_string p "\t</defs>\n\n"); flush p
  |h::tt -> let f, t = if (h.path.reverse == true) then h.path.start, h.path.finish else h.path.finish, h.path.start in
	     output_string p ("\t\t<path id=\""^h.path.pid^"\" d=\"M"^(string_of_float f.x)^","^(string_of_float f.y)^" A25 15 180 0 1 "^(string_of_float t.x)^","^(string_of_float t.y)^"\"/>\n"); printPaths p tt;;

let rec printArcs p l textNos = match l with
    [] -> flush p
  |h::t -> let dir = (if (h.path.reverse==true) then "0 1" else "0 0") in 
    let dashed = if ((compare h.pstyle "dashed") == 0) then " stroke-dasharray=\"3,3\" " else " " in 
      (output_string p ("\t<path d=\"M"^(string_of_float h.path.start.x)^","^(string_of_float h.path.start.y)^" A25 15 180 "^dir^" "^(string_of_float h.path.finish.x)^","^(string_of_float h.path.finish.y)^"\" stroke=\""^h.pcolor^"\""^dashed^"stroke-width=\""^textNos.stroke^"\" fill=\"none\" marker-end=\"url(#"^h.endMarker^")\"/>\n")); printArcs p t textNos;;

let rec printArcTitles p l textNos= match l with
    [] -> flush p
  |h::t -> match h.ttitle with
	"" -> printArcTitles p t textNos
      |_ ->(output_string p ("\t<text font-size=\""^textNos.titleFont^"\" fill=\""^h.tcolor^"\" text-anchor=\"middle\">
\t\t<textPath xlink:href=\"#"^h.tid^"\" startOffset=\"50%\">
\t\t\t<tspan dy=\"-5\">"^h.ttitle^"</tspan>
\t\t</textPath> 
\t</text>\n")); (printArcTitles p t textNos);;


let rec printWords p l textNos = match l with
    [] -> flush p
  |h::t -> 
     let rec pw p info s textNos= match info, s with
	 [], [] -> printWords p t textNos
       |ih::it, sh::st -> (output_string p ("\t<text fill=\"black\" x=\""^(string_of_float sh.x)^"\" y=\""^(string_of_float sh.y)^"\" style=\"font-family:arial; font-size: "^textNos.font^"\">\n\t\t"^ih^"\n\t</text>\n")); pw p it st textNos
       |_, _ -> pw p [] [] textNos
     in (pw p h.wordInfo h.startPoint textNos);;


let printRect p s textNos= (output_string p ("\t<rect style=\"fill:#b3b3b3\" width=\""^textNos.surpWidth^"\" height=\"3\" x=\""^(string_of_float s.sstart.x)^"\" y=\""^(string_of_float s.sstart.y)^"\" />\n
\t<rect style=\"fill:#ff0000\" width=\""^(string_of_float s.width)^"\" height=\"3\" x=\""^(string_of_float s.sstart.x)^"\" y=\""^(string_of_float s.sstart.y)^"\" />\n"));;

let printSurpText p s textNos = (output_string p ("\t<text fill=\"black\" x=\""^(string_of_float s.sstart.x)^"\" y=\""^(string_of_float s.sstart.y)^"\" style=\"font-family:arial; font-size: "^textNos.surpFont^"\">\n\t\t"^s.stext^"\n\t</text>\n"));;

let rec printSurp p l textNos = match l with
    [] -> flush p
  |h::t -> (printRect p h textNos); (printSurpText p h textNos); (printSurp p t textNos);;

let printSVG s p textNos = (output_string p header); (printArrows p s.arrowList textNos); (printPaths p s.arcPathList); (printArcs p s.arcPathList textNos); (printArcTitles p s.arcTitleList textNos); (printWords p s.words textNos); (if ((List.length s.surprisalList) > 0) then (printSurp p s.surprisalList textNos) else (flush p)); (output_string p "\n</svg>"); (flush p);;

let rec makeBigSVG svgList wordl arrows arcs titles surprisals = match svgList with
    [] -> {words=wordl; arrowList=(List.rev arrows); arcPathList=(List.rev arcs); arcTitleList=(List.rev titles); surprisalList=(List.rev surprisals)}
  |h::t -> makeBigSVG t (wordl@h.words) (arrows@h.arrowList) (arcs@h.arcPathList) (titles@h.arcTitleList) (surprisals@h.surprisalList);;


let runSVGPrint svgList onePage textSize = 
  let textNos = (match textSize with
    "small" -> {font="10"; surpFont="8"; surpWidth="25.0"; titleFont="8"; arrowW="4"; arrowH="3"; stroke="2"}
    |"large" -> {font="28"; surpFont="20"; surpWidth="60.0"; titleFont="20"; arrowW="6"; arrowH="3"; stroke="3"}
    |_ -> {font="14"; surpFont="8"; surpWidth="30.0"; titleFont="10"; arrowW="4"; arrowH="3"; stroke="2"}) in 
    match onePage with
    true -> let newSVG = (makeBigSVG svgList [] [] [] [] []) in
	      (printSVG newSVG (makeSVG "s") textNos); true
  |false -> let rec printIt svgList i textNos = match svgList with 
	[] -> true 
      |h::t -> match h.words with
      [] -> (printIt t i textNos)
      |hh::tt -> (printSVG h (makeSVG (string_of_int i)) textNos); (printIt t (i+1) textNos)
    in (printIt svgList 0 textNos);;
