%{
(*Header*)
open YaccTypes;;

let parse_error s =
    print_endline  "Parse error";
    print_endline s;
    flush stdout;;

%}

/* token declarations*/

%token <string> STRING
%token <int> INT
%token <float> FLOAT
%token TAB
%token EOF EOL
%token <string> COLOR
%token DASHED
%token SIZE
%token SENTENCEBEGIN
%token SENTENCEEND
%token WORD
%token ARC
%token SURPRISAL
%type <YaccTypes.sentence list> sentenceList
%start sentenceList

%%
sentenceList: 
  SENTENCEBEGIN EOL wl al SENTENCEEND EOL sentenceList {(makeSentence (List.rev $3) (List.rev $4)) :: $7}
| SENTENCEBEGIN EOL wl al sl SENTENCEEND EOL sentenceList {(makeSentenceSurp (List.rev $3) (List.rev $4) (List.rev $5)) :: $8}
| SENTENCEBEGIN EOL wl al SENTENCEEND EOL {[(makeSentence $3 $4)]}
| SENTENCEBEGIN EOL wl al sl SENTENCEEND EOL {[(makeSentenceSurp $3 $4 $5)]}
| EOF {[makeSentence [] []]}
;

al:
  a al {$1 :: $2}
| a {[$1]}
;


wl:
  w wl {(makeWord $1) :: $2}
| w {[makeWord $1]}
;

sl:
  s sl {$1 :: $2}
| s {[$1]}
;

w:

  WORD TAB STRING w {$3 :: $4}
| TAB STRING w {$2 :: $3}
| WORD TAB STRING EOL {[$3]}
| TAB STRING EOL {[$2]}
;

a:
  ARC TAB INT TAB INT TAB STRING TAB COLOR TAB DASHED EOL {makeArc $3 $5 $7 $9 "DASHED"}
| ARC TAB INT TAB INT TAB STRING TAB DASHED TAB COLOR EOL {makeArc $3 $5 $7 $11 "DASHED"}
| ARC TAB INT TAB INT TAB COLOR TAB STRING TAB DASHED EOL {makeArc $3 $5 $9 $7 "DASHED"}
| ARC TAB INT TAB INT TAB COLOR TAB DASHED TAB STRING EOL {makeArc $3 $5 $11 $7 "DASHED"}
| ARC TAB INT TAB INT TAB DASHED TAB STRING TAB COLOR EOL {makeArc $3 $5 $9 $11 "DASHED"}
| ARC TAB INT TAB INT TAB DASHED TAB COLOR TAB STRING EOL {makeArc $3 $5 $11 $9 "DASHED"}
| ARC TAB INT TAB INT TAB DASHED TAB STRING EOL {makeArc $3 $5 $9 "#000000" "DASHED"}
| ARC TAB INT TAB INT TAB DASHED TAB COLOR EOL {makeArc $3 $5 "" $9 "DASHED"}
| ARC TAB INT TAB INT TAB COLOR TAB STRING EOL {makeArc $3 $5 $9 $7 "SOLID"}
| ARC TAB INT TAB INT TAB COLOR TAB DASHED EOL {makeArc $3 $5 "" $7 "DASHED"}
| ARC TAB INT TAB INT TAB STRING TAB COLOR EOL {makeArc $3 $5 $7 $9 "SOLID"}
| ARC TAB INT TAB INT TAB STRING TAB DASHED EOL {makeArc $3 $5 $7 "#000000" "DASHED"}
| ARC TAB INT TAB INT TAB DASHED EOL {makeArc $3 $5 "" "#000000" "DASHED"}
| ARC TAB INT TAB INT TAB STRING EOL {makeArc $3 $5 $7 "#000000" "DASHED"}
| ARC TAB INT TAB INT TAB COLOR EOL {makeArc $3 $5 "" $7 "SOLID"}
| ARC TAB INT TAB INT EOL {makeArc $3 $5 "" "#000000" "SOLID"}
;
 
s:
SURPRISAL TAB INT TAB FLOAT EOL {makeSurp $3 $5}
;

%%









